-- Performance stuff
local k_drift = k_drift
local k_pogospring = k_pogospring
local MF2_DONTDRAW = MF2_DONTDRAW
local MF_NOCLIPTHING = MF_NOCLIPTHING
local MFE_VERTICALFLIP = MFE_VERTICALFLIP
local MF2_OBJECTFLIP = MF2_OBJECTFLIP
local P_TeleportMove = P_TeleportMove
local P_SetOrigin = P_SetOrigin
local P_MoveOrigin = P_MoveOrigin
local P_SpawnMobj = P_SpawnMobj
local P_KillMobj = P_KillMobj
local FixedMul = FixedMul
local cos = cos
local sin = sin



-- Hat visibility
local cv_hatstoggle = CV_RegisterVar({
	name = "hatvisibility",
	defaultvalue = "On",
	flags = CV_CALL,
	possiblevalue = CV_OnOff,
	func = function(cv)
		if cv.value == 0
			print("Hat visibility has been turned\x85 Off\x80.")
		else
			print("Hat visibility will be turned\x82 On\x80 next race!")
		end
	end
})

freeslot("MT_HAT", "MT_HATMASK")

mobjinfo[MT_HAT] = {
	dispoffset = 1,
	flags = MF_NOGRAVITY|MF_NOCLIP|MF_SCENERY|MF_NOBLOCKMAP|MF_NOCLIPHEIGHT
}

mobjinfo[MT_HATMASK] = {
	dispoffset = -5,
	flags = MF_NOGRAVITY|MF_NOCLIP|MF_SCENERY|MF_NOBLOCKMAP|MF_NOCLIPHEIGHT
}

--Spawn the hat and attach it to a player mobj
addHook("PlayerThink", function(player)
	if not (player.mo and player.mo.valid)
		return
	end
	
	if (cv_hatstoggle.value and player.cos.hat == 0 and player.mo.hatmobj and player.mo.hatmobj.valid)
		P_KillMobj(player.mo.hatmobj)
		player.mo.hatmobj = nil
		return
	end
	
	local hm = player.mo.hatmobj
	if not ((player.cos.hat and not player.mo.hatmobj) or (player.cos.hat and (hm and hm.valid and hm.hatid != player.cos.hat)))
		return
	end
	
	local mo = player.mo
	local hat_id = player.cos.hat
	if hat_id > #Cosmetics.Hats
		CONS_Printf(player, "ERROR: Invalid hat ID, out of range")
		return
	end
	
	local data = Cosmetics.Hats[hat_id]
	local HAT_OBJECT = data.ismask and MT_HATMASK or MT_HAT
	local hm = P_SpawnMobj(mo.x,mo.y,mo.z,HAT_OBJECT)
	mo.hatmobj = hm
	hm.target = mo
	hm.state = data.state
	hm.hatid = hat_id
	return hm
end, MT_PLAYER)

--Workaround to get the consoleplayer's camera mobj... ugh
local consoleplayer_camera = nil
local function getcam(v, player, camera)
    consoleplayer_camera = camera
end
hud.add(getcam, "game")

--Keeping 1.3 custom build compat just in case
local function hatMove(m,x,y,z)
	if P_SetOrigin == nil and P_MoveOrigin == nil then
		P_TeleportMove(m,x,y,z)
	else
		P_MoveOrigin(m,x,y,z)
	end
end


local function hatThinker(mo)

if not cv_hatstoggle.value then P_KillMobj(mo) return end

	local target = mo.target
	if not (target and target.valid and target.player and target.hatmobj == mo)
		P_KillMobj(mo)
		return
	end
	
	local player = target.player
	local height = target.height
	local data = Cosmetics.Hats[mo.hatid]
	
	--Decide to draw the hat or not
	if (target.flags2 & MF2_DONTDRAW)
	or (target.flags & MF_NOCLIPTHING)
	or (consoleplayer_camera == nil)
	or (consoleplayer_camera.chase == false)
		mo.flags2 = $ | MF2_DONTDRAW
	else
		mo.flags2 = $ & ~MF2_DONTDRAW
	end
	
	--Gravity flip stuff
	if target.eflags & MFE_VERTICALFLIP
		height = 0
		mo.eflags = $|MFE_VERTICALFLIP
		mo.flags2 = $|MF2_OBJECTFLIP
	else
		mo.eflags = $ & ~MFE_VERTICALFLIP
		mo.flags2 = $ & ~MF2_OBJECTFLIP
	end
	
	--Track player vars
	if not player.mo then return end
	mo.scale = player.mo.scale
	mo.destscale = player.mo.destscale
    mo.angle = player.frameangle
	
	-- Just making sure kart builds without the rotation and scaling dont freak out.
	if mo.rollangle ~= nil and mo.spritexscale ~= nil and mo.spriteyscale ~= nil then 
		mo.rollangle = (player.mo.rollsum ~= nil and player.mo.rollsum) or player.mo.sloperoll
		mo.spritexscale = player.mo.spritexscale
		mo.spriteyscale = player.mo.spriteyscale
	end
	
	mo.colorized = player.mo.colorized
	
	local color = player.cos.hatcolor
	-- Support for player colorization
	if (color)
		if mo.colorized
			mo.color = player.mo.color
		else
			mo.color = color
		end
		
	else
		mo.color = player.mo.color
	end
	local skinoffset = Cosmetics.SkinOffsets[player.mo.skin] or 0
	local skindriftoffset = Cosmetics.SkinDriftOffsets[player.mo.skin] or 5
	local playerDefinedOffset = player.cos.hat_offset or 0
	if not data.ismask
		
		local drift_x = 0
		local drift_y = 0
		--Standard hat
		--drift offset added here
		if player.kartstuff[k_drift] < 0 or player.kartstuff[k_drift] > 0 and not player.kartstuff[k_pogospring]
			drift_x = cos(player.mo.angle) * skindriftoffset
			drift_y = sin(player.mo.angle) * skindriftoffset
		else
			drift_x = 0
			drift_y = 0
		end
		
		hatMove(mo,target.x - drift_x ,target.y - drift_y ,target.z + height + (skinoffset*mo.scale) + (playerDefinedOffset* mo.scale) + (player.mo.spriteyoffset or 0) )	
	else
		local drift_x = 0
		local drift_y = 0
		if player.kartstuff[k_drift] < 0 or player.kartstuff[k_drift] > 0 and not player.kartstuff[k_pogospring]
			drift_x = cos(player.mo.angle) * skindriftoffset
			drift_y = sin(player.mo.angle) * skindriftoffset
		else
			drift_x = 0
			drift_y = 0
		end
		
		--Masks should be offset a bit...
		local _x = FixedMul(7*mo.scale, cos(mo.angle))
		local _y = FixedMul(7*mo.scale, sin(mo.angle))

		hatMove(mo,(target.x + _x) ,(target.y + _y) ,target.z + height + (skinoffset*mo.scale) + (playerDefinedOffset* mo.scale) + (player.mo.spriteyoffset or 0) )
	end
	
	--Run hat thinker
	if data.thinker
		data.thinker(mo)
	end
end
--Hat thinker, follow the player and set color / angle / flip

addHook("MobjThinker", hatThinker, MT_HAT)
addHook("MobjThinker", hatThinker, MT_HATMASK)
