--Save player config to the data file
local function save(player)
	if player != consoleplayer return end
	
	local file = io.open("client/hatmod.cfg", "w")
	
	--Hat Name
	if player.cos.hat
		file:write("hat \""..Cosmetics.Hats[player.cos.hat].name.."\"\n")
	end
	
	--Hat Color
	if player.cos.hatcolor
		file:write("hatcolor \""..tostring(colornames[player.cos.hatcolor]).."\"\n")
	end
	
	-- Hat Toggle
	if CV_FindVar("hatvisibility").value == 1 or 0
		file:write("hatvisibility \""..CV_FindVar("hatvisibility").value.."\"\n")
	end
	
	--Hat offset
	if player.cos.hat_offset 
		file:write("hatoffset \""..player.cos.hat_offset.."\"\n")
	end
	
	file:write("wait 1\n")
	file:write("echo \"Hat config loaded!\"\n")
	S_StartSound(nil,sfx_kc5e,player)
	io.close(file)
	
	print("Hat config saved.")
end

--Searches the database for stuff based on a string, returns the index or nil
local function search(table, tlen, str)
	for i = 1, tlen
		if table[i].name == str
			return i
		end
	end
end

local function colorsearch(table, tlen, str)
	for i = 1, tlen
		if tostring(table[i]) == str
			return i
		end
	end
end

--Console commands used by the config file.
COM_AddCommand("hat", function(player, hat)
	local s = search(Cosmetics.Hats, #Cosmetics.Hats, hat)
	if (s)
		hat = s
	else
		hat = tonumber(hat)
	end
	if hat == nil or hat < 0 or hat > #Cosmetics.Hats
		if player == consoleplayer
			CONS_Printf(player, "Invalid hat ID")
		end
		return
	end
	player.cos.hat = hat
	local actualhat = Cosmetics.Hats[player.cos.hat]
	
	if (actualhat ~= nil) then
		CONS_Printf(player, "Your current hat is "..actualhat.name.." created by "..actualhat.artist)
	else
		CONS_Printf(player, "No hat for you.")
	end
	
end)

COM_AddCommand("hatcolor", function(player, hatcolor)
	local s = colorsearch(colornames, #colornames, hatcolor)
	if (s)
		hatcolor = s
	else
		hatcolor = tonumber(hatcolor)
	end
	if hatcolor == nil or hatcolor < 0 or hatcolor > #skincolors - 1
		if player == consoleplayer
			CONS_Printf(player, "Invalid hatcolor ID")
		end
		return
	end
	player.cos.hatcolor = hatcolor
	CONS_Printf(player, "Your current hatcolor is "..tostring(colornames[player.cos.hatcolor]))
end)

COM_AddCommand("hatsave", function(player)
save(player)
end)

local function printToPlayer(p, message)
	CONS_Printf(p, message)
    S_StartSound(nil, sfx_radio, p) 
end

COM_AddCommand("hatrandom", function(player)
	player.cos.random_hat = not player.cos.random_hat
	if (player.cos.random_hat) then
		printToPlayer(player, "*You will get a random hat at the start of each race.")
	else 
		printToPlayer(player, "*Random hats off.")
	end
end)

COM_AddCommand("hatoffset", function(player, offset)
	if (not player.cos.init) then 
		printToPlayer(player, "*I don't know how, but you've managed to run this command faster than the addon can set you up. Try again.")
		return 
	end
	if (not offset) then
		if (player.cos.hat_offset ~= nil and player.cos.hat_offset ~= 0) then
			player.cos.hat_offset = 0
			printToPlayer(player, "* Reset offset to \x83zero.")
		else
			printToPlayer(player, "* Gimme a number between -30 and 30 (incl.)")
		end
		return 
	else
		offset = tonumber(offset)
		if offset == nil then
			printToPlayer(player, "* Give me a NUMBER.")
		else
			if (offset < -30 or offset > 30) then
				printToPlayer(player, "* Heh, okay. Between -30 and 30 please.")
				return
			end
			player.cos.hat_offset = offset
			printToPlayer(player, "*Hat offset set to \x83" + offset + ".")
		end
	end
end)

--Player input stuff
addHook("PreThinkFrame", function()
	for player in players.iterate
		--Init
		if not player.cos then
			player.cos = {
				init = true,
				--Settings
				hat = 0,
				random_hat = false,
				hatcolor = 0,
				hat_offset = nil
			}
		end
		
		--Load delay. Should load the cosmetics config each time you rejoin the server
		if player.load_delay == nil
			player.load_delay = TICRATE
		end
		if player.load_delay
			player.load_delay = $ - 1
			if player.load_delay == 10 and player == consoleplayer
				local cosmeticsfile = io.open("client/hatmod.cfg", "r") or io.open("client/cosmetics.cfg", "r")
				if cosmeticsfile then
					for line in cosmeticsfile:lines() do
						if line:sub(1,4) == "wait" or line:sub(1,9) == "soundtest" then -- Srb2 hates when it executes wait or soundtest in buffer.
							continue
						end
						
						COM_BufInsertText(consoleplayer, line)
					end
				end

			end
			return
		end
		
end
end)

local function randomHat()
	local index = P_RandomRange(1, #Cosmetics.Hats)
	return index
end

-- Random Hat
addHook("PlayerThink", function(p)
	if leveltime == 4 then
			if p.cos.init then
				if(p.cos.random_hat) then
					local rdm = randomHat()
					COM_BufInsertText(p, "hat " + rdm)
				end
			end
	end

end)

local function getMaxHats() 
	return FixedInt(FixedCeil(FixedDiv(#Cosmetics.Hats*FRACUNIT, 10*FRACUNIT)))
end

-- List of hats
local function listHats(p, arg)
    arg = tonumber(arg)

	local maxHats = getMaxHats()
	if not arg then
		printToPlayer(p, "\136* Please input a \130number! (Example: hatlist <1-" .. maxHats .. ">)")
	else
		local finalString = "\n\131HAT LIST (Page ".. arg .." of "..maxHats..")\n\n"
		
		if(arg > maxHats or arg <= 0) then 
			printToPlayer(p, "\136* Please enter a \130page\136 number from \1301\136 to \130".. maxHats.."\136.")
			return 
		end
		
		local start_look = 1
		local end_look = (maxHats > 1) and 10 or #Cosmetics.Hats
		if(arg > 1) then
			start_look = (arg*10) - 9
			end_look = start_look + 9
			
			if(end_look > #Cosmetics.Hats) then
				end_look = start_look + (#Cosmetics.Hats - start_look)
			end
		end
		for index=start_look, end_look do
			local hatDescriptorColour = ": \135"
			finalString = finalString .. "\136".. index .. hatDescriptorColour ..Cosmetics.Hats[index].name
			finalString = $ .. "\n"
		end
		
		printToPlayer(p, finalString)
	end
end
COM_AddCommand("hatlist", listHats, 0)
COM_AddCommand("listhat", listHats, 0)